package com.lhsystems.iocc.test;

import java.io.IOException;

import javax.jms.Connection;
import javax.jms.JMSException;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

public class HornetQTC10HADisconnectFailoverReconnection extends HornetQTopicTester {

	private static final String JNP_URL = "jnp://node1:1100,jnp://node2:1100";
	private static final String TOPIC_NAME = "topic/TestTopic1";
	private static final String CONNECTION_FACT_NAME = "/ConnectionFactory";
	private static final String CLIENT_ID_SUB = "TestClientSub5";
	private static final String SUBSCRIPTION_NAME = "TestSubscr5";
	private static final int SENT_MESSAGES_PER_RUN = 10;
	private static final boolean STOP_FOR_INSPECTION = true;
	private static final int SLEEP_BETWEEN_MESSAGES_MILLIS = 200;
	private static final int WAIT_FOR_MESSAGES_SECS = 5;

	private static Logger log = Logger.getLogger(HornetQTC10HADisconnectFailoverReconnection.class);
	private TopicSender sender;

	public static void main(String[] args) {
		HornetQTC10HADisconnectFailoverReconnection testCase = new HornetQTC10HADisconnectFailoverReconnection();
		Connection connection = null;

		try {
			connection = testCase.connect(JNP_URL, CONNECTION_FACT_NAME,
					TOPIC_NAME, CLIENT_ID_SUB, SUBSCRIPTION_NAME);

			testCase.sendMessages(SENT_MESSAGES_PER_RUN);
			log.info("Waiting " + WAIT_FOR_MESSAGES_SECS
					+ "secs for messages to arrive");
			Thread.sleep(WAIT_FOR_MESSAGES_SECS * 1000);

			testCase.closeConnection(connection);
			connection = null;

			stopForInspection("Now check which node the client is connected to and kill -9 the JBoss instance on that node. After node is done, press Enter to continue.");

			testCase.sendMessages(SENT_MESSAGES_PER_RUN);

			log.info("Waiting " + WAIT_FOR_MESSAGES_SECS
					+ "secs for messages to arrive");
			Thread.sleep(WAIT_FOR_MESSAGES_SECS * 1000);
			stopForInspection("Press Enter to continue");

			testCase.sendMessages(SENT_MESSAGES_PER_RUN);
			connection = testCase.connect(JNP_URL, CONNECTION_FACT_NAME,
					TOPIC_NAME, CLIENT_ID_SUB, SUBSCRIPTION_NAME, true);
			log.info("Waiting " + WAIT_FOR_MESSAGES_SECS
					+ "secs for messages to arrive");
			Thread.sleep(WAIT_FOR_MESSAGES_SECS * 1000);
			stopForInspection("Press Enter to continue");

			testCase.closeConnection(connection);
			connection = null;

			testCase.sendMessages(SENT_MESSAGES_PER_RUN);
			connection = testCase.connect(JNP_URL, CONNECTION_FACT_NAME,
					TOPIC_NAME, CLIENT_ID_SUB, SUBSCRIPTION_NAME, false);
			log.info("Waiting " + WAIT_FOR_MESSAGES_SECS
					+ "secs for messages to arrive");
			Thread.sleep(WAIT_FOR_MESSAGES_SECS * 1000);
			stopForInspection("Press Enter to continue");

			log.info("End.\n Test Summary:");
			log.info("SENT messages: " + testCase.getSentMessages());
			log.info("RECEIVED messages: " + testCase.getReceivedMessageCount());
			if (testCase.getSentMessages() != testCase
					.getReceivedMessageCount()) {
				log.error("Sent and received message numbers are different!");
			}
		} catch (JMSException e) {
			log.error("Exception during connection", e);
		} catch (NamingException e) {
			log.error("Exception during lookup", e);
		} catch (InterruptedException e) {
			log.error("Thread was interrupted during sleep", e);
		} catch (IOException e) {
			log.error("Exception reading from console", e);
		} finally {
			try {
				testCase.closeConnection(connection);

				testCase.unsubscribe(JNP_URL, CONNECTION_FACT_NAME,
						CLIENT_ID_SUB, SUBSCRIPTION_NAME);
			} catch (Exception e) {
				log.error("Exception during unsubscribing from topic", e);
			}
		}
	}

	private static void stopForInspection(String message) throws IOException {
		if (STOP_FOR_INSPECTION) {
			log.info(message);
			System.in.read();
		}
	}

	private static void stopForInspection() throws IOException {
		if (STOP_FOR_INSPECTION) {
			log.info("Press Enter to continue");
			System.in.read();
		}
	}

	private void sendMessages(int num) {
		if (sender == null) {
			sender = new TopicSender(SLEEP_BETWEEN_MESSAGES_MILLIS, num,
					JNP_URL, CONNECTION_FACT_NAME, TOPIC_NAME);
		}
		sender.run();
	}

	private int getSentMessages() {
		if (sender == null) {
			throw new IllegalStateException("Sender is not initialized.");
		}
		return sender.getMessageCount();
	}
}
